#define FVDLL_SAVEPARAMETERS

#include "ActiveDiskImpl.h"
#include <UserModules/UserBinder.h>
#include <vector>

// Array of all active disks instances
std::vector<CActiveDisk*> g_instances;

// Initializing the module
FVDLL_API fvdllInitModule(FVDLL_INIT_MODULE* _pInitData)
{
   g_instances.reserve(_pInitData->numContexts);
   return INTEROP_OK;
}

// Initializing a active disk instance
FVDLL_API fvdllInitContext(FVDLL_INIT_CONTEXT* _pInitData, FVDLL_CONTEXT* _outContext)
{
   try
   {
      g_instances.push_back(new CActiveDisk());
      *_outContext = g_instances.back();
      g_instances.back()->SetInitParams(_pInitData->numParameters, _pInitData->pParameters);
      return INTEROP_OK;
   }
   catch(INTEROP_RESULT res)
   {
      return res;
   }
   catch(...)
   {
      return INTEROP_ERROR_GENERIC;
   }
}

// Release all instances
FVDLL_API fvdllReleaseContexts()
{
   try
   {
      for (std::vector<CActiveDisk*>::iterator ppInstance = g_instances.begin(); ppInstance != g_instances.end(); ++ppInstance)
         delete *ppInstance;

      g_instances.clear();
      return INTEROP_OK;
   }
   catch(...)
   {
      return INTEROP_ERROR_GENERIC;
   }
}

FVDLL_API fvdllLoadParameters(FVDLL_CONTEXT _hContext, FVDLL_SAVED_PARAMETERS* _pParams)
{
   try
   {
      reinterpret_cast<CActiveDisk*>(_hContext)->LoadParameters(_pParams->numParameters, _pParams->pParameters);
      return INTEROP_OK;
   }
   catch(INTEROP_RESULT res)
   {
      return res;
   }
   catch(...)
   {
      return INTEROP_ERROR_GENERIC;
   }
}

FVDLL_API fvdllSaveParameters(FVDLL_CONTEXT _hContext)
{
   try
   {
      reinterpret_cast<CActiveDisk*>(_hContext)->SaveParameters();
      return INTEROP_OK;
   }
   catch(INTEROP_RESULT res)
   {
      return res;
   }
   catch(...)
   {
      return INTEROP_ERROR_GENERIC;
   }
}

// Initializing the binder
FVDLL_API fvdllInitBinder(FVDLL_CONTEXT _hContext, FVAPI_BCONDITION _hBCond1, FVAPI_BCONDITION _hBCond2)
{
   try
   {
      reinterpret_cast<CActiveDisk*>(_hContext)->InitDisk(_hBCond1, _hBCond2);
      return INTEROP_OK;
   }
   catch(INTEROP_RESULT res)
   {
      return res;
   }
   catch(...)
   {
      return INTEROP_ERROR_GENERIC;
   }
}

// Before time step calculations
FVDLL_API fvdllBeforeTimeStep(FVDLL_CONTEXT _hContext)
{
   try
   {
      reinterpret_cast<CActiveDisk*>(_hContext)->BeforeTimeStep();
      return INTEROP_OK;
   }
   catch(INTEROP_RESULT res)
   {
      return res;
   }
   catch(...)
   {
      return INTEROP_ERROR_GENERIC;
   }
}

// After time step calculations
FVDLL_API fvdllAfterTimeStep(FVDLL_CONTEXT _hContext)
{
   try
   {
      reinterpret_cast<CActiveDisk*>(_hContext)->AfterTimeStep();
      return INTEROP_OK;
   }
   catch(INTEROP_RESULT res)
   {
      return res;
   }
   catch(...)
   {
      return INTEROP_ERROR_GENERIC;
   }
}

// Getting variable values
FVDLL_API fvdllGetValueInContext(FVDLL_CONTEXT _hContext, FVAPI_VARIABLE _hVariable, FVAPI_CONTEXT _hVarContext, INTEROP_DOUBLE* _outValue)
{
   try
   {
      reinterpret_cast<CActiveDisk*>(_hContext)->GetValueInContext(_hVariable, _hVarContext, _outValue);
      return INTEROP_OK;
   }
   catch(INTEROP_RESULT res)
   {
      return res;
   }
   catch(...)
   {
      return INTEROP_ERROR_GENERIC;
   }
}
