#include "SinusEvaluatorImpl.h"
#include <string>
#include <cmath>
#include <cstring>

using namespace std;

CSinusEvaluator::CSinusEvaluator()
   : m_Function(SINUS), m_Scale(1.0, 1.0, 1.0), m_Amplitude(1.0), m_Frequency(1.0)
{
}

CSinusEvaluator::~CSinusEvaluator()
{
}

void CSinusEvaluator::SetInitParams(INTEROP_COUNT _numParameters, const FVDLL_PARAMETER* _pParameters)
{
   for (INTEROP_COUNT iParam = 0; iParam < _numParameters; ++iParam)
   {
      const FVDLL_PARAMETER & param = _pParameters[iParam];
      if ((strcmp(param.id, "Function") == 0) && (param.type == FVDLL_PARAM_ENUMERATION))
      {
         if (strcmp(param.value.pszEnum, "Sinus") == 0)
         {
            m_Function = SINUS;
         }
         else if (strcmp(param.value.pszEnum, "Cosinus") == 0)
         {
            m_Function = COSINUS;
         }
         else
         {
            throw INTEROP_ERROR_INVALID_PARAMETER;
         }
      }
      else if ((strcmp(param.id, "Parameters/Amplitude") == 0) && (param.type == FVDLL_PARAM_REAL))
      {
         m_Amplitude = param.value.real;
      }
      else if ((strcmp(param.id, "Parameters/Frequency") == 0) && (param.type == FVDLL_PARAM_REAL))
      {
         m_Frequency = param.value.real;
      }
      else if ((strcmp(param.id, "Parameters/Scale/X") == 0) && (param.type == FVDLL_PARAM_REAL))
      {
         m_Scale.X = param.value.real;
      }
      else if ((strcmp(param.id, "Parameters/Scale/Y") == 0) && (param.type == FVDLL_PARAM_REAL))
      {
         m_Scale.Y = param.value.real;
      }
      else if ((strcmp(param.id, "Parameters/Scale/Z") == 0) && (param.type == FVDLL_PARAM_REAL))
      {
         m_Scale.Z = param.value.real;
      }
      else
      {
         throw INTEROP_ERROR_INVALID_PARAMETER;
      }
   }
}

INTEROP_DOUBLE CSinusEvaluator::EvaluateScalar(FVAPI_CONTEXT _hVarContext)
{
   FVAPI_CELL hCell;                // cell descriptor
   INTEROP_DOUBLE centerCoord[3];   // cell center coordinates

   fvapiGetCellOfContext(_hVarContext, &hCell); // Get cell descriptor

   fvapiGetCellCenter(hCell, centerCoord);      // Get cell center coordinates

   // Distance from the origin with considering for the scaling factors
   double distance = sqrt(centerCoord[0] * centerCoord[0] * m_Scale.X * m_Scale.X +
                          centerCoord[1] * centerCoord[1] * m_Scale.Y * m_Scale.Y +
                          centerCoord[2] * centerCoord[2] * m_Scale.Z * m_Scale.Z);

   if (m_Function == CSinusEvaluator::SINUS)
      return sin(distance * m_Frequency) * m_Amplitude;
   else
      return cos(distance * m_Frequency) * m_Amplitude;
}
