#ifndef __C_INTERFACE_H__
#define __C_INTERFACE_H__

#include <Interop/interop_types.h>

#ifndef FV_CALL_CONV
   #ifdef _MSC_VER
      #define FV_CALL_CONV __cdecl
   #else
      #define FV_CALL_CONV __attribute__((__cdecl))
   #endif
#endif

#ifdef _MSC_VER
 #ifdef _FVSOLVER
  #define FV_API __declspec(dllexport) INTEROP_RESULT FV_CALL_CONV
 #else
  #define FV_API __declspec(dllimport) INTEROP_RESULT FV_CALL_CONV
 #endif
#else
 #define FV_API INTEROP_RESULT FV_CALL_CONV
#endif

#ifdef __cplusplus
extern "C" 
{
#endif

/*! \mainpage  API   
*/

/// @defgroup GROUP_DESCRIPTORS   FlowVision
/// @{

typedef INTEROP_HANDLE FVAPI_HANDLE;     ///<  
typedef FVAPI_HANDLE FVAPI_GRID;         ///<   
typedef FVAPI_HANDLE FVAPI_CELL;         ///<   
typedef FVAPI_HANDLE FVAPI_FACE;         ///<   
typedef FVAPI_HANDLE FVAPI_FACET;        ///<   
typedef FVAPI_HANDLE FVAPI_PHASE;        ///<  
typedef FVAPI_HANDLE FVAPI_MODEL;        ///<     (      )
typedef FVAPI_HANDLE FVAPI_PROCESS;      ///<   
typedef FVAPI_HANDLE FVAPI_EQSYS;        ///<    
typedef FVAPI_HANDLE FVAPI_CONTEXT;      ///<    
typedef FVAPI_HANDLE FVAPI_VARIABLE;     ///<  
typedef FVAPI_HANDLE FVAPI_BCONDITION;   ///<    
typedef FVAPI_HANDLE FVAPI_SUBREGION;    ///<  
typedef FVAPI_HANDLE FVAPI_EVALUATOR;    ///<   
typedef INTEROP_INT64 FVAPI_FVMPI_COMM;  ///< FVMPI-

/// @}

// ------------------------------------------------------------------------------------------------------------------------

/// @defgroup GROUP_LOGGING 
/// @{

/// @brief    
enum FVENUM_MGG_TYPE {
   FVAPI_MSG_UNTAGGED,              ///<  
   FVAPI_MSG_INFORMATION,           ///< 
   FVAPI_MSG_WARNING,               ///< 
   FVAPI_MSG_ERROR,                 ///< 
   FVAPI_MSG_CRITICAL_ERROR         ///<  
};

typedef INTEROP_ENUM FVAPI_MSG_TYPE;   ///<   

/// @brief      FlowVision
/// @param[in] _type  
/// @param[in] _message     UTF-8
/// @result INTEROP_OK -  
/// @result INTEROP_ERROR_UNKNOWN_ENTITY -    -
/// @result INTEROP_ERROR_GENERIC -   
FV_API fvapiLogMessage(FVAPI_MSG_TYPE _type, INTEROP_STRING _message);

/// @}

// ------------------------------------------------------------------------------------------------------------------------

/// @defgroup GROUP_LICENSE  
/// @{

/// @brief       
/// @param[in] _libGUID GUID  
/// @param[in] _randInt  
/// @param[out] _hashOut   -
/// @result INTEROP_OK -  
/// @result INTEROP_ERROR_INVALID_PARAMETER -   GUID'
/// @result INTEROP_ERROR_GUID_NOT_FOUND - GUID  
FV_API fvapiCheckAuthenticity(INTEROP_STRING _libGUID, INTEROP_UINT64 _randInt, INTEROP_STRING _hashOut);

/// @}

// ------------------------------------------------------------------------------------------------------------------------

/// @defgroup GROUP_STATUS     
/// @{

/// @brief   
/// @param[out] _pTime  
/// @result INTEROP_OK -  
FV_API fvapiGetCurrentTime(INTEROP_DOUBLE* _pTime);

/// @}
// ------------------------------------------------------------------------------------------------------------------------

/// @defgroup GROUP_MPI   FVMPI
/// @{

/// @brief  FVMPI-
/// @param[out] _pFvMPIComm FVMPI-
/// @result INTEROP_OK -  
FV_API fvapiGetFvMPIComm(FVAPI_FVMPI_COMM* _pFvMPIComm);

/// @}
// ------------------------------------------------------------------------------------------------------------------------

/// @defgroup GROUP_CELLS   
/// @{

/// @brief   
/// @param[in] _hCell  
/// @param[out] _pVolume  
/// @result INTEROP_OK -  
/// @result INTEROP_ERROR_INVALID_PARAMETER -     (  NULL)
/// @result INTEROP_ERROR_GENERIC -     
FV_API fvapiGetCellVolume(FVAPI_CELL _hCell, INTEROP_DOUBLE* _pVolume); 

/// @brief   
/// @param[in] _hCell  
/// @param[out] _pCenter      
/// @result INTEROP_OK -  
/// @result INTEROP_ERROR_INVALID_PARAMETER -     (  NULL)
/// @result INTEROP_ERROR_GENERIC -     
FV_API fvapiGetCellCenter(FVAPI_CELL _hCell, INTEROP_DOUBLE* _pCenter); 

/// @brief    
/// @param[in] _hCell  
/// @param[out] _phSubregion  
/// @result INTEROP_OK -  
/// @result INTEROP_ERROR_INVALID_PARAMETER -     (  NULL)
/// @result INTEROP_ERROR_GENERIC -     
FV_API fvapiGetCellSubregion(FVAPI_CELL _hCell, FVAPI_SUBREGION* _phSubregion);

/// @brief      
/// @param[in] _hCell  
/// @param[out] _phModel   
/// @result INTEROP_OK -  
/// @result INTEROP_ERROR_INVALID_PARAMETER -     (  NULL)
/// @result INTEROP_ERROR_GENERIC -     
FV_API fvapiGetCellModel(FVAPI_CELL _hCell, FVAPI_MODEL* _phModel);

/// @brief    
/// @param[in] _hCell  
/// @param[out] _pFaceCount   
/// @result INTEROP_OK -  
/// @result INTEROP_ERROR_INVALID_PARAMETER -     (  NULL)
/// @result INTEROP_ERROR_GENERIC -     
FV_API fvapiGetCellFaceCount(FVAPI_CELL _hCell, INTEROP_COUNT* _pFaceCount);

/// @brief     
/// @param[in] _hCell  
/// @param[in] _index  
/// @param[out] _phFace  
/// @result INTEROP_OK -  
/// @result INTEROP_ERROR_INVALID_PARAMETER -     (  NULL)
/// @result INTEROP_ERROR_OUT_OF_RANGE -    
/// @result INTEROP_ERROR_GENERIC -     
FV_API fvapiGetCellFace(FVAPI_CELL _hCell, INTEROP_INDEX _index, FVAPI_FACE* _phFace);

/// @}

// ------------------------------------------------------------------------------------------------------------------------

/// @defgroup GROUP_FACES  
/// @{

/// @brief    
/// @param[in] _hFace  
/// @param[in] _hSide  ,   
/// @param[out] _pFaceArea   
/// @result INTEROP_OK -  
/// @result INTEROP_ERROR_INVALID_PARAMETER -     (  NULL)
/// @result INTEROP_ERROR_GENERIC -     
/// @remarks                 .
FV_API fvapiGetFaceArea(FVAPI_FACE _hFace, FVAPI_CELL _hSide, INTEROP_DOUBLE* _pFaceArea);

/// @brief    
/// @param[in] _hFace  
/// @param[in] _hSide  ,   
/// @param[out] _pFaceCenter       
/// @result INTEROP_OK -  
/// @result INTEROP_ERROR_INVALID_PARAMETER -     (  NULL)
/// @result INTEROP_ERROR_GENERIC -     
/// @remarks             .
FV_API fvapiGetFaceCenter(FVAPI_FACE _hFace, FVAPI_CELL _hSide, INTEROP_DOUBLE* _pFaceCenter);

/// @brief    
/// @param[in] _hFace  
/// @param[in] _hSide  ,   
/// @param[out] _pFaceNormal       
/// @result INTEROP_OK -  
/// @result INTEROP_ERROR_INVALID_PARAMETER -     (  NULL)
/// @result INTEROP_ERROR_GENERIC -     
/// @remarks            ,    .
FV_API fvapiGetFaceNormal(FVAPI_FACE _hFace, FVAPI_CELL _hSide, INTEROP_DOUBLE* _pFaceNormal);

/// @brief    
/// @param[in] _hFace  
/// @param[in] _hSide  ,   
/// @param[out] _phNeighbor      
/// @result INTEROP_OK -  
/// @result INTEROP_ERROR_INVALID_PARAMETER -     (  NULL)
/// @result INTEROP_ERROR_GENERIC -     
/// @remarks             .
FV_API fvapiGetFaceNeighbor(FVAPI_FACE _hFace, FVAPI_CELL _hSide, FVAPI_CELL* _phNeighbor);

/// @brief   ,  
/// @param[in] _hFace  
/// @param[out] _phFacet  
/// @result INTEROP_OK -  
/// @result INTEROP_ERROR_INVALID_PARAMETER -     (  NULL)
/// @result INTEROP_ERROR_GENERIC -     
/// @remarks       ,    NULL.
FV_API fvapiGetFaceFacet(FVAPI_FACE _hFace, FVAPI_FACET* _phFacet);

/// @brief       
/// @param[in] _hFace  
/// @param[in] _hSide  ,   
/// @param[out] _phBCondition    
/// @result INTEROP_OK -  
/// @result INTEROP_ERROR_INVALID_PARAMETER -     (  NULL)
/// @result INTEROP_ERROR_GENERIC -     
/// @remarks       ,        ,  FlowVision .
FV_API fvapiGetFaceBCondition(FVAPI_FACE _hFace, FVAPI_CELL _hSide, FVAPI_BCONDITION* _phBCondition);

/// @}

// ------------------------------------------------------------------------------------------------------------------------

/// @defgroup GROUP_SUBREGIONS    
/// @{

/// @brief      
/// @param[in] _hSubregion  
/// @param[out] _phModel   
/// @result INTEROP_OK -  
/// @result INTEROP_ERROR_INVALID_PARAMETER -     (  NULL)
/// @result INTEROP_ERROR_GENERIC -     
FV_API fvapiGetSubregionModel(FVAPI_CELL _hSubregion, FVAPI_MODEL* _phModel);

/// @brief      
/// @param[in] _hSubregion  
/// @param[out] _pCount     
/// @result INTEROP_OK -  
/// @result INTEROP_ERROR_INVALID_PARAMETER -     (  NULL)
/// @result INTEROP_ERROR_GENERIC -     
/// @remarks       (  ,  FlowVision ).
FV_API fvapiGetSubregionBConditionCount(FVAPI_CELL _hSubregion, INTEROP_COUNT* _pCount);

/// @brief       
/// @param[in] _hSubregion  
/// @param[in] _index   
/// @param[out] _phBCondition    
/// @result INTEROP_OK -  
/// @result INTEROP_ERROR_INVALID_PARAMETER -     (  NULL)
/// @result INTEROP_ERROR_OUT_OF_RANGE -    
/// @result INTEROP_ERROR_GENERIC -     
/// @remarks       (  ,  FlowVision ).
FV_API fvapiGetSubregionBCondition(FVAPI_CELL _hSubregion, INTEROP_INDEX _index, FVAPI_BCONDITION* _phBCondition);

/// @brief     
/// @param[in] _hSubregion  
/// @param[out] _phGrid   
/// @result INTEROP_OK -  
/// @result INTEROP_ERROR_INVALID_PARAMETER -     (  NULL)
/// @result INTEROP_ERROR_GENERIC -     
FV_API fvapiGetSubregionGrid(FVAPI_CELL _hSubregion, FVAPI_GRID* _phGrid);

/// @brief      
/// @param[in] _hBCondition    
/// @param[out] _phSubregion  
/// @result INTEROP_OK -  
/// @result INTEROP_ERROR_INVALID_PARAMETER -     (  NULL)
/// @result INTEROP_ERROR_GENERIC -     
FV_API fvapiGetBConditionSubregion(FVAPI_CELL _hBCondition, FVAPI_SUBREGION* _phSubregion);

/// @}

// ------------------------------------------------------------------------------------------------------------------------

/// @defgroup GROUP_MODELS   
/// @{

/// @brief   ,    
/// @param[in] _hModel   
/// @param[out] _pPhaseCount  
/// @result INTEROP_OK -  
/// @result INTEROP_ERROR_INVALID_PARAMETER -     (  NULL)
/// @result INTEROP_ERROR_GENERIC -     
FV_API fvapiGetModelPhaseCount(FVAPI_MODEL _hModel, INTEROP_UINT32* _pPhaseCount);

/// @brief    ,    
/// @param[in] _hModel   
/// @param[in] _index    
/// @param[out] _phPhase  
/// @result INTEROP_OK -  
/// @result INTEROP_ERROR_INVALID_PARAMETER -     (  NULL)
/// @result INTEROP_ERROR_OUT_OF_RANGE -    
/// @result INTEROP_ERROR_GENERIC -     
FV_API fvapiGetModelPhase(FVAPI_MODEL _hModel, INTEROP_UINT32 _index, FVAPI_PHASE* _phPhase);

/// @brief      
/// @param[in] _hModel  
/// @param[in,out] _pNumBytes      (.  )
/// @param[out] _pName     UTF-8
/// @result INTEROP_OK -  
/// @result INTEROP_ERROR_INVALID_PARAMETER -     (  NULL)
/// @result INTEROP_ERROR_INSUFFICIENT_BUFFER -      
/// @result INTEROP_ERROR_GENERIC -     
/// @remarks   _pName   NULL,         _pNumBytes   INTEROP_OK.
/// @remarks   _pName   NULL,   _pNumBytes      .   ,   INTEROP_ERROR_INSUFFICIENT_BUFFER.
FV_API fvapiGetModelName(FVAPI_MODEL _hModel, INTEROP_COUNT* _pNumBytes, INTEROP_STRING _pName);

/// @}

// ------------------------------------------------------------------------------------------------------------------------

/// @defgroup GROUP_PHASES ,     
/// @{

/// @brief     
enum FVENUM_STD_PROCESS {
   FVAPI_MOTION_PROCESS,            ///< 
   FVAPI_TURBULENCE_PROCESS,        ///< 
   FVAPI_HEAT_TRANSFER_PROCESS,     ///< 
   FVAPI_RADIATION_PROCESS,         ///<  
   FVAPI_MASS_TRANSFER_PROCESS,     ///< 
   FVAPI_PHASE_TRANSFER_PROCESS,    ///<   
   FVAPI_ELECTRO_DYN_PROCESS        ///< 
};

typedef INTEROP_ENUM FVAPI_STD_PROCESS;   ///<    

#define FVAPI_VARIABLE_VEL             "VEL"             ///< 
#define FVAPI_VARIABLE_PRES            "PRES"            ///< 
#define FVAPI_VARIABLE_VISCOSITY       "VISCOSITY"       ///< 
#define FVAPI_VARIABLE_DENSITY         "DENSITY"         ///< 
#define FVAPI_VARIABLE_TEMP            "TEMP"            ///< 
#define FVAPI_VARIABLE_ROT_VEL         "ROT_VEL"         ///< 
#define FVAPI_VARIABLE_STRESS          "STRESS"          ///<  
#define FVAPI_VARIABLE_DIV_VEL         "DIV_VEL"         ///<  
#define FVAPI_VARIABLE_TOTAL_PRESSURE  "TOTAL_PRESSURE"  ///<  
#define FVAPI_VARIABLE_CP              "CP"              ///<  

/// @brief     
/// @param[in] _id    
/// @param[out] _phProcess   
/// @result INTEROP_OK -  
/// @result INTEROP_ERROR_INVALID_PARAMETER -     (  NULL)
/// @result INTEROP_ERROR_UNKNOWN_ENTITY -   
/// @result INTEROP_ERROR_GENERIC -   
FV_API fvapiGetStdProcess(FVAPI_STD_PROCESS _id, FVAPI_PROCESS* _phProcess);

/// @brief        
/// @param[in] _hPhase  
/// @param[in] _hProcess   
/// @param[out] _phEqSys    
/// @result INTEROP_OK -  
/// @result INTEROP_ERROR_INVALID_PARAMETER -     (  NULL)
/// @result INTEROP_ERROR_GENERIC -     
FV_API fvapiGetPhaseEqSys(FVAPI_PHASE _hPhase, FVAPI_PROCESS _hProcess, FVAPI_EQSYS* _phEqSys);

/// @brief    
/// @param[in] _hPhase  
/// @param[in,out] _pNumBytes      (.  )
/// @param[out] _pName     UTF-8
/// @result INTEROP_OK -  
/// @result INTEROP_ERROR_INVALID_PARAMETER -     (  NULL)
/// @result INTEROP_ERROR_INSUFFICIENT_BUFFER -      
/// @result INTEROP_ERROR_GENERIC -     
/// @remarks   _pName   NULL,         _pNumBytes   INTEROP_OK.
/// @remarks   _pName   NULL,   _pNumBytes      .   ,   INTEROP_ERROR_INSUFFICIENT_BUFFER.
FV_API fvapiGetPhaseName(FVAPI_PHASE _hPhase, INTEROP_COUNT* _pNumBytes, INTEROP_STRING _pName);

/// @brief       
/// @param[in] _hProcess   
/// @param[in] _id   ,     
/// @param[in] _hPhase  ,  NULL,    
/// @param[out] _phVariable   (   NULL)
/// @result INTEROP_OK -  
/// @result INTEROP_ERROR_INVALID_PARAMETER -     (  NULL)
/// @result INTEROP_ERROR_GENERIC -     
/// @remarks          NULL   _phVariable   INTEROP_OK.
FV_API fvapiRequestVariable(FVAPI_PROCESS _hProcess, INTEROP_STRING _id, FVAPI_PHASE _hPhase, FVAPI_VARIABLE* _phVariable);

/// @brief  ,    
/// @param[in] _hVariable  
/// @param[out] _pIsVector INTEROP_TRUE,   , INTEROP_FALSE   
/// @result INTEROP_OK -  
/// @result INTEROP_ERROR_INVALID_PARAMETER -     (  NULL)
/// @result INTEROP_ERROR_GENERIC -     
FV_API fvapiIsVariableVector(FVAPI_VARIABLE _hVariable, INTEROP_BOOL* _pIsVector);

/// @brief    
/// @param[in] _hVariable  
/// @param[out] _phPhase  
/// @result INTEROP_OK -  
/// @result INTEROP_ERROR_INVALID_PARAMETER -     (  NULL)
/// @result INTEROP_ERROR_GENERIC -     
/// @remarks   ,          NULL.
FV_API fvapiGetVariablePhase(FVAPI_VARIABLE _hVariable, FVAPI_PHASE* _phPhase);

/// @brief     
/// @param[in] _hVariable  
/// @param[out] _phProcess   
/// @result INTEROP_OK -  
/// @result INTEROP_ERROR_INVALID_PARAMETER -     (  NULL)
/// @result INTEROP_ERROR_GENERIC -     
/// @remarks            NULL.
FV_API fvapiGetVariableProcess(FVAPI_VARIABLE _hVariable, FVAPI_PROCESS* _phProcess);

/// @}

// ------------------------------------------------------------------------------------------------------------------------

/// @defgroup GROUP_LOCAL_VALUES    
/// @{

/// @brief      
enum FVENUM_COMPONENT {
   FVAPI_COMPONENT_SOURCE = -2,     ///<  
   FVAPI_COMPONENT_LENGTH = -1,     ///< 
   FVAPI_COMPONENT_X = 0,           ///< X
   FVAPI_COMPONENT_Y = 1,           ///< Y
   FVAPI_COMPONENT_Z = 2            ///< Z
};

typedef INTEROP_ENUM FVAPI_COMPONENT;  ///<       

/// @brief   
enum FVENUM_INTERPOLATION {
   FVAPI_NO_INTERPOLATION,          ///<  
   FVAPI_BASIC_INTERPOLATION        ///<  
};

typedef INTEROP_ENUM FVAPI_INTERPOLATION; ///<  

/// @brief      
/// @param[in] _hModel  
/// @param[in] _hContext    
/// @param[in] _hVariable  
/// @param[out] _pResult   (   3-     )
/// @result INTEROP_OK -  
/// @result INTEROP_ERROR_INVALID_PARAMETER -     (  NULL)
FV_API fvapiGetVariableValue(FVAPI_MODEL _hModel, FVAPI_CONTEXT _hContext, FVAPI_VARIABLE _hVariable, INTEROP_DOUBLE* _pResult);

/// @brief      
/// @param[in] _hCell  
/// @param[in] _hVariable  
/// @param[out] _pResult   (   3-     )
/// @result INTEROP_OK -  
/// @result INTEROP_ERROR_INVALID_PARAMETER -     (  NULL)
FV_API fvapiGetVariableValueInCell(FVAPI_CELL _hCell, FVAPI_VARIABLE _hVariable, INTEROP_DOUBLE* _pResult);

/// @brief            
/// @param[in] _hGrid   
/// @param[in] _hVariable  
/// @param[in] _component     (    )
/// @param[in] _interpolation   
/// @param[in] _numPoints  
/// @param[in] _pCoords    ( _numPoints*3)
/// @param[out] _pValues    (   _numPoints      _numPoints*3    )
/// @result INTEROP_OK -  
/// @result INTEROP_ERROR_INVALID_PARAMETER -     (  NULL)
/// @result INTEROP_ERROR_UNKNOWN_ENTITY -    -
/// @result INTEROP_ERROR_OUT_OF_RANGE -    FlowVision   (INT_MAX)
/// @result INTEROP_ERROR_GENERIC -          FlowVision
/// @remarks    ,        .
/// @remarks        ,      .
FV_API fvapiGetLocalValues_p(FVAPI_GRID _hGrid, FVAPI_VARIABLE _hVariable, FVAPI_COMPONENT _component, FVAPI_INTERPOLATION _interpolation, INTEROP_COUNT _numPoints, const INTEROP_DOUBLE* _pCoords, INTEROP_DOUBLE* _pValues); 

/// @brief             
/// @param[in] _hGrid   
/// @param[in] _numVariables  
/// @param[in] _phVariables    ( _numVariables)
/// @param[in] _pComponents    ( _numVariables),   NULL -        FVAPI_COMPONENT_SOURCE
/// @param[in] _interpolation   
/// @param[in] _numPoints  
/// @param[in] _pCoords    ( _numPoints*3)
/// @param[out] _ppValues       ( _numVariables,      _numPoints      _numPoints*3    )
/// @result INTEROP_OK -  
/// @result INTEROP_ERROR_INVALID_PARAMETER -     (  NULL)
/// @result INTEROP_ERROR_UNKNOWN_ENTITY -    -
/// @result INTEROP_ERROR_OUT_OF_RANGE -    FlowVision     (INT_MAX)
/// @result INTEROP_ERROR_GENERIC -          FlowVision
/// @remarks    ,        .
/// @remarks        ,      .
FV_API fvapiMultiGetLocalValues_p(FVAPI_GRID _hGrid, INTEROP_COUNT _numVariables, const FVAPI_VARIABLE* _phVariables, const FVAPI_COMPONENT* _pComponents, FVAPI_INTERPOLATION _interpolation, INTEROP_COUNT _numPoints, const INTEROP_DOUBLE* _pCoords, INTEROP_DOUBLE* const* _ppValues); 

/// @brief  ,  
/// @result  ,          
FV_API fvapiGetUndefinedValue(INTEROP_DOUBLE *_pUndefinedValue);

/// @}

// ------------------------------------------------------------------------------------------------------------------------

/// @defgroup GROUP_INTEGRAL_VALUES   
/// @{

typedef struct FVAPI_SURFACE_INTEGRAL
{
   INTEROP_COUNT size;
   INTEROP_DOUBLE area;
   INTEROP_DOUBLE intgByArea;
   INTEROP_DOUBLE vecIntgByArea[3];
   INTEROP_DOUBLE intgByMassFlowIn;
   INTEROP_DOUBLE intgByMassFlowOut;
} FVAPI_SURFACE_INTEGRAL;

typedef struct FVAPI_BOUNDARY_INTEGRAL
{
   INTEROP_COUNT  size;
   INTEROP_DOUBLE scalarInt;           // <f>   (   )
   INTEROP_DOUBLE vectorInt[3];        //  
   INTEROP_DOUBLE avrFlowPlus;         // <f>        
   INTEROP_DOUBLE avrFlowMinus;        // <f>        
   INTEROP_DOUBLE area;                //  
   INTEROP_DOUBLE massFlowPlus;        //       
   INTEROP_DOUBLE massFlowMinus;       //       
   INTEROP_DOUBLE volumeFlowPlus;      //      
   INTEROP_DOUBLE volumeFlowMinus;     //      
   INTEROP_DOUBLE heatFlux;            //    
   INTEROP_DOUBLE forceWF[3];          // 
   INTEROP_DOUBLE momentWF[3];         // 
   INTEROP_DOUBLE flow_Plus;           //     
   INTEROP_DOUBLE flow_Minus;          //     
   INTEROP_DOUBLE sigmaByFlow;         //   
   INTEROP_DOUBLE sigmaByArea;         //   
} FVAPI_BOUNDARY_INTEGRAL;


/// @brief       
/// @param[in] _numBoundaries  
/// @param[in] _phBoundaries      ( _numBoundaries)
/// @param[in] _hVariable  
/// @param[in] _component     (    )
/// @param[out] _pResults 
/// @result INTEROP_OK -  
/// @result INTEROP_ERROR_INVALID_PARAMETER -     (  NULL)
/// @result INTEROP_ERROR_INVALID_OPERATION -     
/// @result INTEROP_ERROR_UNKNOWN_ENTITY -    -
/// @result INTEROP_ERROR_GENERIC -          FlowVision
/// @remarks    ,        .
/// @remarks       .
/// @remarks          ,      .
FV_API fvapiGetSurfaceIntegralValues_p(INTEROP_UINT32 _numBoundaries, const FVAPI_BCONDITION* _phBoundaries, FVAPI_VARIABLE _hVariable, INTEROP_ENUM _component, FVAPI_SURFACE_INTEGRAL* _pResults);

/// @brief       
/// @param[in] _hVariable  
/// @param[in] _component     (    )
/// @param[in] _hSubregion  
/// @param[in] _hBCond   
/// @param[in] _momentPt    
/// @param[in] _gridCoordSys       
/// @param[out] _pResults 
/// @result INTEROP_OK -  
/// @result INTEROP_ERROR_INVALID_PARAMETER -     (  NULL)
/// @result INTEROP_ERROR_INVALID_OPERATION -     
/// @result INTEROP_ERROR_UNKNOWN_ENTITY -    -
/// @result INTEROP_ERROR_GENERIC -          FlowVision
/// @remarks    ,        .
/// @remarks       .
/// @remarks          ,      .
FV_API fvapiGetBoundaryCharacteristics_p(FVAPI_VARIABLE _hVariable, INTEROP_ENUM _component, FVAPI_SUBREGION _hSubregion, FVAPI_BCONDITION _hBCond, INTEROP_DOUBLE* _momentPt, ENUM_INTEROP_BOOL _gridCoordSys,
                                         FVAPI_BOUNDARY_INTEGRAL* _pResults);
/// @}

// ------------------------------------------------------------------------------------------------------------------------

/// @defgroup GROUP_VARIABLE_CONTEXT       
/// @{

/// @brief       
/// @param[in] _hContext    
/// @param[out] _hCell  
/// @result INTEROP_OK -  
/// @result INTEROP_ERROR_INVALID_PARAMETER -     (  NULL)
/// @result INTEROP_ERROR_GENERIC -     
FV_API fvapiGetCellOfContext(FVAPI_CONTEXT _hContext, FVAPI_CELL* _hCell);

/// @brief       
/// @param[in] _hContext    
/// @param[out] _hFace  
/// @result INTEROP_OK -  
/// @result INTEROP_ERROR_INVALID_PARAMETER -     (  NULL)
/// @result INTEROP_ERROR_GENERIC -     
FV_API fvapiGetFaceOfContext(FVAPI_CONTEXT _hContext, FVAPI_FACE* _hFace);

/// @}

// ------------------------------------------------------------------------------------------------------------------------

/// @defgroup GROUP_SAVE_PARAMETERS     
/// @{

/// @brief       
/// @param[in] _id   
/// @param[in] _value  
/// @result INTEROP_OK -  
/// @result INTEROP_ERROR_INVALID_OPERATION -       
/// @remarks  API-     fvdllSaveParameters()  
FV_API fvapiSaveParamReal(INTEROP_STRING _id, INTEROP_DOUBLE _value);

/// @brief       
/// @param[in] _id   
/// @param[in] _value  
/// @result INTEROP_OK -  
/// @result INTEROP_ERROR_INVALID_OPERATION -       
/// @remarks  API-     fvdllSaveParameters()  
FV_API fvapiSaveParamInteger(INTEROP_STRING _id, INTEROP_INT32 _value);

/// @brief      
/// @param[in] _id   
/// @param[in] _value  
/// @result INTEROP_OK -  
/// @result INTEROP_ERROR_INVALID_OPERATION -       
/// @remarks  API-     fvdllSaveParameters()  
FV_API fvapiSaveParamByte(INTEROP_STRING _id, INTEROP_UINT8 _value);

/// @brief      
/// @param[in] _id   
/// @param[in] _value  
/// @result INTEROP_OK -  
/// @result INTEROP_ERROR_INVALID_OPERATION -       
/// @remarks  API-     fvdllSaveParameters()  
FV_API fvapiSaveParamBoolean(INTEROP_STRING _id, INTEROP_BOOL _value);

/// @brief      
/// @param[in] _id   
/// @param[in] _value  
/// @result INTEROP_OK -  
/// @result INTEROP_ERROR_INVALID_OPERATION -       
/// @remarks  API-     fvdllSaveParameters()  
FV_API fvapiSaveParamString(INTEROP_STRING _id, INTEROP_STRING _value);

/// @}

// ------------------------------------------------------------------------------------------------------------------------

#ifdef __cplusplus
} // extern "C" 
#endif

#endif // ifndef __C_INTERFACE_H__
