#ifndef __USERMODULES_H__
#define __USERMODULES_H__

#include <Interop/interop_types.h>
#include <Interop/interop_codes.h>
#include <Interface/c_interface.h>

#ifdef _MSC_VER
 #define FVDLL_API __declspec(dllexport) INTEROP_RESULT FV_CALL_CONV
#else
 #define FVDLL_API INTEROP_RESULT FV_CALL_CONV
#endif

#ifdef __cplusplus
extern "C" 
{
#endif

/// @brief ,    
typedef struct FVDLL_PARAMETER_FILE
{
   INTEROP_BYTE*           data;       ///<     
   INTEROP_COUNT           size;       ///<    
} FVDLL_PARAMETER_FILE;

typedef FVDLL_PARAMETER_FILE FVDLL_BINARY_DATA;

/// @brief      
enum FVENUM_PARAMETER_TYPES 
{
   FVDLL_PARAM_REAL,                   ///<  
   FVDLL_PARAM_EVALUATOR,              ///< 
   FVDLL_PARAM_INTEGER,                ///<  
   FVDLL_PARAM_BYTE,                   ///< 
   FVDLL_PARAM_BOOLEAN,                ///<  
   FVDLL_PARAM_STRING,                 ///< 
   FVDLL_PARAM_ENUMERATION,            ///<   
   FVDLL_PARAM_FILE                    ///<   
};

/// @brief     
typedef union FVDLL_PARAMETER_VALUE
{
   INTEROP_DOUBLE          real;       ///<    
   FVAPI_EVALUATOR         hEval;      ///<  
   INTEROP_INT32           integer;    ///< 32-  
   INTEROP_UINT8           byte;       ///< 8-      
   INTEROP_BOOL            boolean;    ///< 8-  
   INTEROP_STRING          pszString;  ///<   ,  
   INTEROP_STRING          pszEnum;    ///<     ,  
   FVDLL_PARAMETER_FILE*   pFile;      ///<    ,    
} FVDLL_PARAMETER_VALUE;

/// @brief   
typedef struct FVDLL_PARAMETER
{
   INTEROP_STRING          id;         ///<   
   FVENUM_PARAMETER_TYPES  type;       ///<  
   FVDLL_PARAMETER_VALUE   value;      ///<  
} FVDLL_PARAMETER;

/// @brief     
typedef struct FVDLL_INIT_MODULE
{
   INTEROP_COUNT size;                 ///<    
   INTEROP_COUNT numContexts;          ///<   
} FVDLL_INIT_MODULE;

/// @brief     
typedef INTEROP_HANDLE FVDLL_CONTEXT;

/// @brief     
typedef struct FVDLL_INIT_CONTEXT
{
   INTEROP_COUNT size;                 ///<    
   INTEROP_COUNT numParameters;        ///<  
   const FVDLL_PARAMETER* pParameters; ///<  
} FVDLL_INIT_CONTEXT;

typedef FVDLL_INIT_CONTEXT FVDLL_SAVED_PARAMETERS;

/// @brief   
/// @param[in] _pInitData   
/// @result  
FVDLL_API fvdllInitModule(FVDLL_INIT_MODULE* _pInitData);

/// @brief   
/// @param[in] _pInitData   
/// @param[out] _outContext   
/// @result  
/// @remarks           (     ).
/// @remarks        FlowVision     .
FVDLL_API fvdllInitContext(FVDLL_INIT_CONTEXT* _pInitData, FVDLL_CONTEXT* _outContext);

/// @brief    
/// @result  
/// @remarks               .
FVDLL_API fvdllReleaseContexts();


#ifdef FVDLL_SAVEPARAMETERS

/// @brief     
/// @param[in] _hContext   
/// @param[in] _pParams  
/// @result  
FVDLL_API fvdllLoadParameters(FVDLL_CONTEXT _hContext, FVDLL_SAVED_PARAMETERS* _pParams);

/// @brief    
/// @result  
/// @remarks       API- fvapiSaveParamReal(), fvapiSaveParamInteger()  ..
FVDLL_API fvdllSaveParameters(FVDLL_CONTEXT _hContext);

#endif


#ifdef FVDLL_SAVEBINARYDATA

/// @brief      
/// @param[in] _hContext   
/// @param[in] _pBinData  
/// @result  
FVDLL_API fvdllLoadBinaryData(FVDLL_CONTEXT _hContext, FVDLL_BINARY_DATA* _pBinData);

/// @brief     
/// @param[in] _hContext   
/// @param[out] _outBinData  
/// @result  
FVDLL_API fvdllSaveBinaryData(FVDLL_CONTEXT _hContext, FVDLL_BINARY_DATA** _outBinData);

/// @brief  ,    
/// @param[in] _hContext   
/// @result  
FVDLL_API fvdllFreeBinaryDataBuffer();

#endif


#ifdef __cplusplus
} // extern "C" 
#endif

#endif // ifndef __USERMODULES_H__
